# encoding: UTF-8
#--
#
#################################################################################
# Copyright 2010-2015 Free Software's Seed
#
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas por la Comisión Europea– 
# versiones posteriores de la EUPL (la «Licencia»);
# Solo podrá usarse esta obra si se respeta la Licencia.
# Puede obtenerse una copia de la Licencia en:
#
# http://www.osor.eu/eupl/european-union-public-licence-eupl-v.1.1
#
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye «TAL CUAL»,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones que establece la Licencia.
#################################################################################
#
#++


class IngresoController < ApplicationController

  def index
    redirect_to :action => :listado 
  end

  # en agentes: establece los parametros de ordenación
  def ordenado
    session[:ingreso_asc_desc] = params[:asc_desc] ? params[:asc_desc] : "ASC"
    session[:ingreso_orden] = params[:orden] ? params[:orden] : "fecha"
    redirect_to :action => :listado
  end

  # en agentes: establece los parametros de filtro
  def filtrado
    session[:ingreso_filtro_etapa] = params[:filtro][:etapa] if params[:filtro][:etapa]
    session[:ingreso_filtro_partida] = params[:filtro][:partida] if params[:filtro][:partida]
    session[:ingreso_filtro_ref_contable] = params[:filtro][:ref_contable] if params[:filtro][:ref_contable]
    redirect_to :action => :listado
  end

  def elementos_filtrado
    session[:ingreso_asc_desc] ||= "ASC"
    session[:ingreso_orden] ||= "fecha"
    session[:ingreso_filtro_partida] ||= "todas"
    session[:ingreso_filtro_etapa] ||= "todas"
    session[:ingreso_filtro_ref_contable] ||= ""

    @etapa = @agente.etapa.find_by_id( session[:ingreso_filtro_etapa] ) unless session[:ingreso_filtro_etapa] == "todas"
    @partida = PartidaIngreso.find_by_id( session[:ingreso_filtro_partida] ) unless session[:ingreso_filtro_partida] == "todas"

    datos_formulario
    filtro_etapa = [[_("Todas"), "todas"]] + @agente.etapa.order(:fecha_inicio).collect{|p| [p.nombre, p.id]} 
    filtro_partida = [[_("Todas"), "todas"]] + PartidaIngreso.order(:nombre).collect{|t| [t.nombre, t.id]}
    
    @opciones_filtrado = [ {rotulo: _("Seleccione etapa"), nombre: "etapa", opciones: filtro_etapa},
                           {rotulo: _("Seleccione partida"), nombre: "partida", opciones: filtro_partida},
                           {rotulo: _("Seleccione ref.contable"), :nombre => "ref_contable", :tipo => "texto"} ]
    @accion_filtrado = {:action => :filtrado, :listado => :listado}
    @estado_filtrado = [ (@etapa ? @etapa.nombre : _("Todas las etapas")), (@partida ? @partida.nombre : _("Todas las partidas de ingreso")) ]
  end


	# en agentes: lista los ingresos
  def listado
    elementos_filtrado 
    condiciones = {} 
    condiciones[:fecha] = @etapa.fecha_inicio..@etapa.fecha_fin if @etapa
    condiciones[:partida_ingreso_id] = @partida.id if @partida
    condiciones_ref_contable = nil if session[:ingreso_filtro_ref_contable].blank?
    condiciones_ref_contable = ["ref_contable LIKE ?", session[:ingreso_filtro_ref_contable]] unless session[:ingreso_filtro_ref_contable].blank?

    @ingresos = @paginado = @agente.ingreso.where(condiciones_ref_contable).paginate(:conditions => condiciones,
		:order => (session[:ingreso_orden] + " " + session[:ingreso_asc_desc] ),
		:page => (params[:format]=='xls' ? nil : params[:page]), :per_page => (params[:format_xls_count] || session[:por_pagina]) )

    @formato_xls = @ingresos.total_entries
    respond_to do |format|
      format.html
      format.xls do
        @tipo = "ingreso"
        @objetos = @ingresos
        nom_fich = "ingresos_" + Time.now.strftime("%Y-%m-%d")
        render 'comunes_xls/listado', :xls => nom_fich, :layout => false
      end
    end
  end

	# en convocatorias: prepara el formulario de crear o editar
  def editar_nuevo
    @ingreso = @agente.ingreso.find_by_id(params[:id]) || Ingreso.new(agente_id: @agente.id)
    datos_formulario
    render (:update) { |page| page.formulario(:partial => "formulario", :update => params[:update]) }
  end

	# en agentes: modifica o crea un proveedor
  def modificar_crear
    @ingreso = @agente.ingreso.find_by_id(params[:id]) || Ingreso.new(agente_id: @agente.id)
    @ingreso.update_attributes params[:ingreso]
    if @ingreso.errors.empty?
      # Si es uno ya existente, modifica la linea
      render(:update) { |page|   page.modificar :update => params[:update], :partial => "ingreso" , :mensaje => { :errors => @ingreso.errors } } if params[:id]
      # Si es uno nuevo lo incluye en la parte superior del listado
      render :update do |page|
        page.show "nuevos_ingresos"
        page.modificar :update => "ingreso_nuevo_" + params[:i], :partial => "nuevo_ingreso", :mensaje => { :errors => @ingreso.errors }
        page.replace "formulario_anadir_anadir", :inline => "<%= anadir(:url => {:action => 'editar_nuevo', :i => (params[:i].to_i + 1).to_s, :update => 'formulario_anadir'}) %>" unless params[:id]
      end unless params[:id] 
    else
      datos_formulario
      render(:update) { |page| page.recargar_formulario :partial => "formulario", :mensaje => {:errors => @ingreso.errors} }
    end 
  end

	# en agentes: elimina un proveedor
  def eliminar
    @ingreso = @agente.ingreso.find_by_id(params[:id])
    @ingreso.destroy if @ingreso
    render (:update) {|page| page.eliminar :update => params[:update], :mensaje => {:errors => @ingreso.errors, :eliminar => true}}
  end

 private
  def datos_formulario
    @partidas = PartidaIngreso.order("nombre").collect{|p| [p.nombre, p.id.to_s]}
    @monedas = @agente.moneda.collect {|a| [a.abreviatura, a.id]}
    @financiadores = Agente.where(financiador: true).order("nombre").collect{|f| [f.nombre, f.id.to_s]}
    @proyectos = @agente.proyecto_implementador.collect{|f| [f.nombre, f.id.to_s]}
  end

end

